---
jupytext:
  formats: ipynb,md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.15.2
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# in eine Lösung Fehler einbauen

Sitzung 2024-01-24

+++

gegeben: eine Lösung
    
gesucht: eine kaputte Lösung, die man Studierenden zur Reparatur vorlegen kann.

(ja, das ist eine Aufgabe auf einem höheren Level).

+++

## Aufgabe mit Lösung

gegeben:
* ein String, z.B. `s = "Hallo Echo! Hallo Otto!"
* eine Liste `tokens_start_stop` von Tupeln, die jeweils den Start und das Ende eines Teilstrings angeben.

```{code-cell} ipython3
s = "Hallo Echo! Hallo Otto!"
p = "01234567890123456789012"
s[0:5]
s[6:10]
```

```{code-cell} ipython3
tokens_start_stop = [(0,4), (6,9), (12,16), (18,21)]
```

gesucht: eine Schleife, die zur Liste `tokens` die zugehörigen Teilstrings in einer Liste sammelt.

Korrekte Lösung:

```{code-cell} ipython3
# Vorbereitung
t_strings = []
for x in tokens_start_stop:
    print(x)
```

```{code-cell} ipython3
# konventionell
t_strings = []
for start, stop in tokens_start_stop:
    t_strings.append( s[start:stop+1] )
t_strings
```

```{code-cell} ipython3
# oder als List Comprehension
t_strings = [ s[start:stop+1] for (start, stop) in tokens_start_stop ]
t_strings
```

## Lösung kaputtisieren

```{code-cell} ipython3
t_strings = {}
for start, stop in tokens_start_stop:
    t_strings.append( s[start:stop] )
t_strings
```

## Typ von Klausuraufgabe: kaputte Lösung reparieren

+++

gegeben:
* ein String, 
  * z.B. `s = "Hallo Echo! Hallo Otto!"`
* eine Liste `tokens_start_stop` von Tupeln, die jeweils den Start und das Ende eines Teilstrings angeben.
  * z.B. `tokens_start_stop = [(0,4), (6,9), (12,16), (18,21)]`

gweünschte Ausgabe:

eine "Lösung", die unvollstänig ist, und auch noch einen Fehler enthält.

+++

**Aufgabe**: korrigieren und ergänzen Sie die fehlerhafte Lösung!

```{code-cell} ipython3
tokens_start_stop = [(0,4), (6,9), (12,16), (18,21)]
s = "Hallo Echo! Hallo Otto!"

def tokens(string, start_stop_list):
    to_return = {}
    for start, stop in start_stop_list:
        ...
    return to_return
```

```{code-cell} ipython3
# tokens(s, tokens_start_stop) == ['Hallo', 'Echo', 'Hallo', 'Otto']
```

```{code-cell} ipython3
tokens(s, tokens_start_stop)
```

```{code-cell} ipython3
# auch eine Lösung

s = "Hallo Echo! Hallo Otto!"
tokens_start_stop = [(0,4), (6,9), (12,16), (18,21)]

def tokens(string, start_stop_list):
    to_return = {}
    for start, stop in start_stop_list:
        to_return[(start,stop)] = string[start:stop+1]
    return [ x for x in to_return.values() ]
tokens(s, tokens_start_stop)
```

## misc

Da bin ich auch schon 'drauf 'reingefallen:

```{code-cell} ipython3
# gleicher Typ bei index und slice
s = "Hallo"
s[1], s[1:3]
```

```{code-cell} ipython3
# unterschiedlicher Typ bei index und slice!
s = list("Hallo")
s[1], s[1:3]
```

```{code-cell} ipython3
s = "Hallo"
for x in range(len(s)):
    print(s[x:x+2])
```

```{code-cell} ipython3
s = list("Hallo")
for x in range(len(s)):
    print(s[x:x+3])
```
